// Copyright 2022 Esri
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//   https://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

import SwiftUI

extension SamplesApp {
    /// The common samples for this app.
    static let commonSamples: [Sample] = [
        Add3DTilesLayer(),
        AddENCExchangeSet(),
        AddWMSLayer(),
        AddCustomDynamicEntityDataSource(),
        AddDynamicEntityLayer(),
        AddFeatureLayers(),
        AddFeaturesWithContingentValues(),
        AddRasterFromFile(),
        AddRasterFromService(),
        AddSceneLayerFromService(),
        AnalyzeNetworkWithSubnetworkTrace(),
        Animate3DGraphic(),
        AnimateImagesWithImageOverlay(),
        ApplyMosaicRuleToRasters(),
        ApplyScheduledUpdatesToPreplannedMapArea(),
        ApplyUniqueValueRenderer(),
        AuthenticateWithOAuth(),
        BrowseOGCAPIFeatureService(),
        BrowseBuildingFloors(),
        ChangeCameraController(),
        ChangeMapViewBackground(),
        ChangeViewpoint(),
        ClipGeometry(),
        ConfigureBasemapStyleParameters(),
        ConfigureClusters(),
        CreateAndEditGeometries(),
        CreateAndSaveKML(),
        CreateBuffersAroundPoints(),
        CreateConvexHullAroundGeometries(),
        CreateConvexHullAroundPoints(),
        CreateDynamicBasemapGallery(),
        CreateLoadReport(),
        CreateMobileGeodatabase(),
        CreatePlanarAndGeodeticBuffers(),
        CreateSymbolStylesFromWebStyles(),
        CutGeometry(),
        DensifyAndGeneralizeGeometry(),
        DisplayAnnotation(),
        DisplayClusters(),
        DisplayContentOfUtilityNetworkContainer(),
        DisplayDimensions(),
        DisplayMap(),
        DisplayMapFromMobileMapPackage(),
        DisplayMapFromPortalItem(),
        DisplayOverviewMap(),
        DisplayScene(),
        DisplaySceneFromMobileScenePackage(),
        DisplayWebSceneFromPortalItem(),
        DownloadPreplannedMapArea(),
        DownloadVectorTilesToLocalCache(),
        EditAndSyncFeaturesWithFeatureService(),
        EditFeatureAttachments(),
        EditFeaturesWithFeatureLinkedAnnotation(),
        EditGeodatabaseWithTransactions(),
        EditWithBranchVersioning(),
        FilterByDefinitionExpressionOrDisplayFilter(),
        FilterFeaturesInScene(),
        FindAddressWithReverseGeocode(),
        FindClosestFacilityFromPoint(),
        FindClosestFacilityToMultiplePoints(),
        FindNearestVertex(),
        FindRoute(),
        FindRouteAroundBarriers(),
        FindRouteInMobileMapPackage(),
        FindRouteInTransportNetwork(),
        GenerateOfflineMap(),
        GenerateOfflineMapWithCustomParameters(),
        GenerateOfflineMapWithLocalBasemap(),
        GeocodeOffline(),
        GetElevationAtPointOnSurface(),
        GroupLayersTogether(),
        IdentifyKMLFeatures(),
        IdentifyFeaturesInWMSLayer(),
        IdentifyGraphics(),
        IdentifyLayerFeatures(),
        IdentifyRasterCell(),
        ListSpatialReferenceTransformations(),
        ManageBookmarks(),
        ManageOperationalLayers(),
        MatchViewpointOfGeoViews(),
        MeasureDistanceInScene(),
        MonitorChangesToDrawStatus(),
        MonitorChangesToLayerViewState(),
        MonitorChangesToMapLoadStatus(),
        NavigateRoute(),
        NavigateRouteWithRerouting(),
        OrbitCameraAroundObject(),
        PlayKMLTour(),
        ProjectGeometry(),
        QueryFeatureTable(),
        QueryFeaturesWithArcadeExpression(),
        QueryRelatedFeatures(),
        RenderMultilayerSymbols(),
        RunValveIsolationTrace(),
        SearchForWebMap(),
        SearchWithGeocode(),
        SelectFeaturesInFeatureLayer(),
        SelectFeaturesInSceneLayer(),
        SetBasemap(),
        SetFeatureRequestMode(),
        SetMaxExtent(),
        SetMinAndMaxScale(),
        SetReferenceScale(),
        SetSurfaceNavigationConstraint(),
        SetSurfacePlacementMode(),
        SetUpLocationDrivenGeotriggers(),
        SetViewpointRotation(),
        SetVisibilityOfSubtypeSublayer(),
        ShowCallout(),
        ShowCoordinatesInMultipleFormats(),
        ShowDeviceLocation(),
        ShowDeviceLocationHistory(),
        ShowDeviceLocationWithNMEADataSources(),
        ShowExtrudedFeatures(),
        ShowGrid(),
        ShowLabelsOnLayer(),
        ShowLineOfSightBetweenPoints(),
        ShowMobileMapPackageExpirationDate(),
        ShowPopup(),
        ShowRealisticLightAndShadows(),
        ShowResultOfSpatialOperations(),
        ShowResultOfSpatialRelationships(),
        ShowScaleBar(),
        ShowServiceArea(),
        ShowUtilityAssociations(),
        ShowViewshedFromGeoelementInScene(),
        ShowViewshedFromPointInScene(),
        ShowViewshedFromPointOnMap(),
        SnapGeometryEdits(),
        StyleFeaturesWithCustomDictionary(),
        StyleGraphicsWithRenderer(),
        StyleGraphicsWithSymbols(),
        StylePointWithDistanceCompositeSceneSymbol(),
        StylePointWithPictureMarkerSymbols(),
        StylePointWithSceneSymbol(),
        StyleSymbolsFromMobileStyleFile(),
        TraceUtilityNetwork(),
        ValidateUtilityNetworkTopology()
    ]
    
    /// The iOS samples for this app.
    static let iOSSamples: [Sample] = {
#if !targetEnvironment(macCatalyst) && !targetEnvironment(simulator)
        // Exclude AR samples from Mac Catalyst and Simulator targets
        // as they don't have camera and sensors available.
        [
            AugmentRealityToCollectData(),
            AugmentRealityToFlyOverScene(),
            AugmentRealityToNavigateRoute(),
            AugmentRealityToShowHiddenInfrastructure(),
            AugmentRealityToShowTabletopScene()
        ]
#else
        []
#endif
    }()
}

// MARK: Sample Structs


struct Add3DTilesLayer: Sample {
    var name: String { "Add 3D tiles layer" }
    var category: String { "Scenes" }
    var description: String { "Add a layer to visualize 3D tiles data that conforms to the OGC 3D Tiles specification." }
    var snippets: [String] { ["Add3DTilesLayerView.swift"] }
    var tags: Set<String> { ["3d tiles", "OGC", "OGC API", "layers", "scene", "service"] }
    
    func makeBody() -> AnyView { .init(Add3DTilesLayerView()) }
}

struct AddENCExchangeSet: Sample {
    var name: String { "Add ENC exchange set" }
    var category: String { "Layers" }
    var description: String { "Display nautical charts per the ENC specification." }
    var snippets: [String] { ["AddENCExchangeSetView.swift"] }
    var tags: Set<String> { ["ENC", "data", "hydrographic", "layers", "maritime", "nautical chart"] }
    var hasDependencies: Bool { true }

    func makeBody() -> AnyView { .init(AddENCExchangeSetView()) }
}

struct AddWMSLayer: Sample {
    var name: String { "Add WMS layer" }
    var category: String { "Layers" }
    var description: String { "Display a WMS layer using a WMS service URL." }
    var snippets: [String] { ["AddWMSLayerView.swift"] }
    var tags: Set<String> { ["OGC", "web map service"] }
    
    func makeBody() -> AnyView { .init(AddWMSLayerView()) }
}

struct AddCustomDynamicEntityDataSource: Sample {
    var name: String { "Add custom dynamic entity data source" }
    var category: String { "Layers" }
    var description: String { "Create a custom dynamic entity data source and display it using a dynamic entity layer." }
    var snippets: [String] { ["AddCustomDynamicEntityDataSourceView.swift", "AddCustomDynamicEntityDataSourceView.Vessel.swift"] }
    var tags: Set<String> { ["data", "dynamic", "entity", "label", "labeling", "live", "real-time", "stream", "track"] }
    var hasDependencies: Bool { true }

    func makeBody() -> AnyView { .init(AddCustomDynamicEntityDataSourceView()) }
}

struct AddDynamicEntityLayer: Sample {
    var name: String { "Add dynamic entity layer" }
    var category: String { "Layers" }
    var description: String { "Display data from an ArcGIS stream service using a dynamic entity layer." }
    var snippets: [String] { ["AddDynamicEntityLayerView.swift", "AddDynamicEntityLayerView.Model.swift", "AddDynamicEntityLayerView.SettingsView.swift", "AddDynamicEntityLayerView.VehicleCallout.swift"] }
    var tags: Set<String> { ["data", "dynamic", "entity", "live", "purge", "real-time", "service", "stream", "track"] }
    
    func makeBody() -> AnyView { .init(AddDynamicEntityLayerView()) }
}

struct AddFeatureLayers: Sample {
    var name: String { "Add feature layers" }
    var category: String { "Layers" }
    var description: String { "Display feature layers from various data sources." }
    var snippets: [String] { ["AddFeatureLayersView.swift"] }
    var tags: Set<String> { ["feature", "geodatabase", "geopackage", "layers", "service", "shapefile", "table"] }
    var hasDependencies: Bool { true }

    func makeBody() -> AnyView { .init(AddFeatureLayersView()) }
}

struct AddFeaturesWithContingentValues: Sample {
    var name: String { "Add features with contingent values" }
    var category: String { "Edit and Manage Data" }
    var description: String { "Create and add features whose attribute values satisfy a predefined set of contingencies." }
    var snippets: [String] { ["AddFeaturesWithContingentValuesView.swift", "AddFeaturesWithContingentValuesView.Model.swift", "AddFeaturesWithContingentValuesView.AddFeatureView.swift"] }
    var tags: Set<String> { ["coded values", "contingent values", "feature table", "geodatabase"] }
    var hasDependencies: Bool { true }

    func makeBody() -> AnyView { .init(AddFeaturesWithContingentValuesView()) }
}

struct AddRasterFromFile: Sample {
    var name: String { "Add raster from file" }
    var category: String { "Layers" }
    var description: String { "Create and use a raster layer made from a local raster file." }
    var snippets: [String] { ["AddRasterFromFileView.swift"] }
    var tags: Set<String> { ["data", "image", "import", "layer", "raster", "visualization"] }
    var hasDependencies: Bool { true }

    func makeBody() -> AnyView { .init(AddRasterFromFileView()) }
}

struct AddRasterFromService: Sample {
    var name: String { "Add raster from service" }
    var category: String { "Layers" }
    var description: String { "Create a raster layer from a raster image service." }
    var snippets: [String] { ["AddRasterFromServiceView.swift"] }
    var tags: Set<String> { ["image service", "raster"] }
    
    func makeBody() -> AnyView { .init(AddRasterFromServiceView()) }
}

struct AddSceneLayerFromService: Sample {
    var name: String { "Add scene layer from service" }
    var category: String { "Layers" }
    var description: String { "Display an ArcGIS scene layer from a service." }
    var snippets: [String] { ["AddSceneLayerFromServiceView.swift"] }
    var tags: Set<String> { ["3D", "URL", "buildings", "model", "scene", "service"] }
    
    func makeBody() -> AnyView { .init(AddSceneLayerFromServiceView()) }
}

struct AnalyzeNetworkWithSubnetworkTrace: Sample {
    var name: String { "Analyze network with subnetwork trace" }
    var category: String { "Utility Networks" }
    var description: String { "Get a server-defined trace configuration for a given tier and modify its traversability scope, add new condition barriers, and control what is included in the subnetwork trace result." }
    var snippets: [String] { ["AnalyzeNetworkWithSubnetworkTraceView.swift", "AnalyzeNetworkWithSubnetworkTraceView.Model.swift"] }
    var tags: Set<String> { ["category comparison", "condition barriers", "network analysis", "network attribute comparison", "subnetwork trace", "trace configuration", "traversability", "utility network", "validate consistency"] }
    
    func makeBody() -> AnyView { .init(AnalyzeNetworkWithSubnetworkTraceView()) }
}

struct Animate3DGraphic: Sample {
    var name: String { "Animate 3D graphic" }
    var category: String { "Scenes" }
    var description: String { "An `OrbitGeoElementCameraController` follows a graphic while the graphic's position and rotation are animated." }
    var snippets: [String] { ["Animate3DGraphicView.swift", "Animate3DGraphicView.Model.swift", "Animate3DGraphicView.SettingsView.swift"] }
    var tags: Set<String> { ["animation", "camera", "heading", "pitch", "roll", "rotation", "visualize"] }
    var hasDependencies: Bool { true }

    func makeBody() -> AnyView { .init(Animate3DGraphicView()) }
}

struct AnimateImagesWithImageOverlay: Sample {
    var name: String { "Animate images with image overlay" }
    var category: String { "Visualization" }
    var description: String { "Animate a series of images with an image overlay." }
    var snippets: [String] { ["AnimateImagesWithImageOverlayView.swift"] }
    var tags: Set<String> { ["3d", "animation", "drone", "dynamic", "image frame", "image overlay", "real time", "rendering"] }
    var hasDependencies: Bool { true }

    func makeBody() -> AnyView { .init(AnimateImagesWithImageOverlayView()) }
}

struct ApplyMosaicRuleToRasters: Sample {
    var name: String { "Apply mosaic rule to rasters" }
    var category: String { "Layers" }
    var description: String { "Apply mosaic rule to a mosaic dataset of rasters." }
    var snippets: [String] { ["ApplyMosaicRuleToRastersView.swift"] }
    var tags: Set<String> { ["image service", "mosaic method", "mosaic rule", "raster"] }
    
    func makeBody() -> AnyView { .init(ApplyMosaicRuleToRastersView()) }
}

struct ApplyScheduledUpdatesToPreplannedMapArea: Sample {
    var name: String { "Apply scheduled updates to preplanned map area" }
    var category: String { "Edit and Manage Data" }
    var description: String { "Apply scheduled updates to a downloaded preplanned map area." }
    var snippets: [String] { ["ApplyScheduledUpdatesToPreplannedMapAreaView.swift"] }
    var tags: Set<String> { ["offline", "pre-planned", "preplanned", "synchronize", "update"] }
    var hasDependencies: Bool { true }

    func makeBody() -> AnyView { .init(ApplyScheduledUpdatesToPreplannedMapAreaView()) }
}

struct ApplyUniqueValueRenderer: Sample {
    var name: String { "Apply unique value renderer" }
    var category: String { "Visualization" }
    var description: String { "Render features in a layer using a distinct symbol for each unique attribute value." }
    var snippets: [String] { ["ApplyUniqueValueRendererView.swift"] }
    var tags: Set<String> { ["draw", "renderer", "symbol", "symbology", "values"] }
    
    func makeBody() -> AnyView { .init(ApplyUniqueValueRendererView()) }
}
@available(macCatalyst, unavailable)
struct AugmentRealityToCollectData: Sample {
    var name: String { "Augment reality to collect data" }
    var category: String { "Augmented Reality" }
    var description: String { "Tap on real-world objects to collect data." }
    var snippets: [String] { ["AugmentRealityToCollectDataView.swift"] }
    var tags: Set<String> { ["attachment", "augmented reality", "capture", "collection", "collector", "data", "field", "field worker", "full-scale", "mixed reality", "survey", "world-scale"] }
    
    func makeBody() -> AnyView { .init(AugmentRealityToCollectDataView()) }
}
@available(macCatalyst, unavailable)
struct AugmentRealityToFlyOverScene: Sample {
    var name: String { "Augment reality to fly over scene" }
    var category: String { "Augmented Reality" }
    var description: String { "Use augmented reality (AR) to quickly explore a scene more naturally than you could with a touch or mouse interface." }
    var snippets: [String] { ["AugmentRealityToFlyOverSceneView.swift"] }
    var tags: Set<String> { ["augmented reality", "bird\'s eye", "birds-eye-view", "fly over", "flyover", "mixed reality", "translation factor"] }
    
    func makeBody() -> AnyView { .init(AugmentRealityToFlyOverSceneView()) }
}
@available(macCatalyst, unavailable)
struct AugmentRealityToNavigateRoute: Sample {
    var name: String { "Augment reality to navigate route" }
    var category: String { "Augmented Reality" }
    var description: String { "Use a route displayed in the real world to navigate." }
    var snippets: [String] { ["AugmentRealityToNavigateRouteView.swift", "AugmentRealityToNavigateRouteView.ARSceneView.swift"] }
    var tags: Set<String> { ["augmented reality", "directions", "full-scale", "guidance", "mixed reality", "navigate", "navigation", "real-scale", "route", "routing", "world-scale"] }
    
    func makeBody() -> AnyView { .init(AugmentRealityToNavigateRouteView()) }
}
@available(macCatalyst, unavailable)
struct AugmentRealityToShowHiddenInfrastructure: Sample {
    var name: String { "Augment reality to show hidden infrastructure" }
    var category: String { "Augmented Reality" }
    var description: String { "Visualize hidden infrastructure in its real-world location using augmented reality." }
    var snippets: [String] { ["AugmentRealityToShowHiddenInfrastructureView.swift", "AugmentRealityToShowHiddenInfrastructureView.ARSceneView.swift"] }
    var tags: Set<String> { ["augmented reality", "full-scale", "infrastructure", "lines", "mixed reality", "pipes", "real-scale", "underground", "visualization", "visualize", "world-scale"] }
    
    func makeBody() -> AnyView { .init(AugmentRealityToShowHiddenInfrastructureView()) }
}
@available(macCatalyst, unavailable)
struct AugmentRealityToShowTabletopScene: Sample {
    var name: String { "Augment reality to show tabletop scene" }
    var category: String { "Augmented Reality" }
    var description: String { "Use augmented reality (AR) to pin a scene to a table or desk for easy exploration." }
    var snippets: [String] { ["AugmentRealityToShowTabletopSceneView.swift"] }
    var tags: Set<String> { ["augmented reality", "drop", "mixed reality", "model", "pin", "place", "table-top", "tabletop"] }
    var hasDependencies: Bool { true }

    func makeBody() -> AnyView { .init(AugmentRealityToShowTabletopSceneView()) }
}

struct AuthenticateWithOAuth: Sample {
    var name: String { "Authenticate with OAuth" }
    var category: String { "Cloud and Portal" }
    var description: String { "Authenticate with ArcGIS Online (or your own portal) using OAuth2 to access secured resources (such as private web maps or layers)." }
    var snippets: [String] { ["AuthenticateWithOAuthView.swift"] }
    var tags: Set<String> { ["OAuth", "authentication", "cloud", "credential", "portal", "security"] }
    
    func makeBody() -> AnyView { .init(AuthenticateWithOAuthView()) }
}

struct BrowseOGCAPIFeatureService: Sample {
    var name: String { "Browse OGC API feature service" }
    var category: String { "Layers" }
    var description: String { "Browse an OGC API feature service for layers and add them to the map." }
    var snippets: [String] { ["BrowseOGCAPIFeatureServiceView.swift"] }
    var tags: Set<String> { ["OGC", "OGC API", "browse", "catalog", "feature", "layers", "service", "web"] }
    
    func makeBody() -> AnyView { .init(BrowseOGCAPIFeatureServiceView()) }
}

struct BrowseBuildingFloors: Sample {
    var name: String { "Browse building floors" }
    var category: String { "Maps" }
    var description: String { "Display and browse through building floors from a floor-aware web map." }
    var snippets: [String] { ["BrowseBuildingFloorsView.swift"] }
    var tags: Set<String> { ["building", "facility", "floor", "floor-aware", "floors", "ground floor", "indoor", "level", "site", "story"] }
    
    func makeBody() -> AnyView { .init(BrowseBuildingFloorsView()) }
}

struct ChangeCameraController: Sample {
    var name: String { "Change camera controller" }
    var category: String { "Scenes" }
    var description: String { "Control the behavior of the camera in a scene." }
    var snippets: [String] { ["ChangeCameraControllerView.swift"] }
    var tags: Set<String> { ["3D", "camera controller", "orbit", "tracking"] }
    var hasDependencies: Bool { true }

    func makeBody() -> AnyView { .init(ChangeCameraControllerView()) }
}

struct ChangeMapViewBackground: Sample {
    var name: String { "Change map view background" }
    var category: String { "Maps" }
    var description: String { "Customize map view's background by changing its grid properties." }
    var snippets: [String] { ["ChangeMapViewBackgroundView.swift", "ChangeMapViewBackgroundView.Model.swift", "ChangeMapViewBackgroundView.SettingsView.swift"] }
    var tags: Set<String> { ["background", "grid", "map"] }
    
    func makeBody() -> AnyView { .init(ChangeMapViewBackgroundView()) }
}

struct ChangeViewpoint: Sample {
    var name: String { "Change viewpoint" }
    var category: String { "Maps" }
    var description: String { "Set the map view to a new viewpoint." }
    var snippets: [String] { ["ChangeViewpointView.swift"] }
    var tags: Set<String> { ["animate", "extent", "pan", "rotate", "scale", "view", "zoom"] }
    
    func makeBody() -> AnyView { .init(ChangeViewpointView()) }
}

struct ClipGeometry: Sample {
    var name: String { "Clip geometry" }
    var category: String { "Edit and Manage Data" }
    var description: String { "Clip a geometry with another geometry." }
    var snippets: [String] { ["ClipGeometryView.swift"] }
    var tags: Set<String> { ["analysis", "clip", "geometry"] }
    
    func makeBody() -> AnyView { .init(ClipGeometryView()) }
}

struct ConfigureBasemapStyleParameters: Sample {
    var name: String { "Configure basemap style parameters" }
    var category: String { "Maps" }
    var description: String { "Apply basemap style parameters customization for a basemap, such as displaying all labels in a specific language or displaying every label in their corresponding local language." }
    var snippets: [String] { ["ConfigureBasemapStyleParametersView.swift"] }
    var tags: Set<String> { ["basemap style", "language", "language strategy", "map", "point", "viewpoint"] }
    
    func makeBody() -> AnyView { .init(ConfigureBasemapStyleParametersView()) }
}

struct ConfigureClusters: Sample {
    var name: String { "Configure clusters" }
    var category: String { "Layers" }
    var description: String { "Add client side feature reduction on a point feature layer that is not pre-configured with clustering." }
    var snippets: [String] { ["ConfigureClustersView.swift", "ConfigureClustersView.Model.swift", "ConfigureClustersView.SettingsView.swift"] }
    var tags: Set<String> { ["aggregate", "bin", "cluster", "group", "merge", "normalize", "popup", "reduce", "renderer", "summarize"] }
    
    func makeBody() -> AnyView { .init(ConfigureClustersView()) }
}

struct CreateAndEditGeometries: Sample {
    var name: String { "Create and edit geometries" }
    var category: String { "Maps" }
    var description: String { "Use the Geometry Editor to create new point, multipoint, polyline, or polygon geometries or to edit existing geometries by interacting with a map view." }
    var snippets: [String] { ["CreateAndEditGeometriesView.swift"] }
    var tags: Set<String> { ["draw", "edit", "freehand", "geometry editor", "sketch", "vertex"] }
    
    func makeBody() -> AnyView { .init(CreateAndEditGeometriesView()) }
}

struct CreateAndSaveKML: Sample {
    var name: String { "Create and save KML file" }
    var category: String { "Edit and Manage Data" }
    var description: String { "Construct a KML document and save it as a KMZ file." }
    var snippets: [String] { ["CreateAndSaveKMLView.swift", "CreateAndSaveKMLView.Model.swift", "CreateAndSaveKMLView.Views.swift"] }
    var tags: Set<String> { ["KML", "KMZ", "Keyhole", "OGC"] }
    
    func makeBody() -> AnyView { .init(CreateAndSaveKMLView()) }
}

struct CreateBuffersAroundPoints: Sample {
    var name: String { "Create buffers around points" }
    var category: String { "Analysis" }
    var description: String { "Generate multiple individual buffers or a single unioned buffer around multiple points." }
    var snippets: [String] { ["CreateBuffersAroundPointsView.swift"] }
    var tags: Set<String> { ["analysis", "buffer", "geometry", "planar"] }
    
    func makeBody() -> AnyView { .init(CreateBuffersAroundPointsView()) }
}

struct CreateConvexHullAroundGeometries: Sample {
    var name: String { "Create convex hull around geometries" }
    var category: String { "Analysis" }
    var description: String { "Generate convex hull polygon(s) from multiple input geometries." }
    var snippets: [String] { ["CreateConvexHullAroundGeometriesView.swift"] }
    var tags: Set<String> { ["analysis", "geometry", "outline", "perimeter", "union"] }
    
    func makeBody() -> AnyView { .init(CreateConvexHullAroundGeometriesView()) }
}

struct CreateConvexHullAroundPoints: Sample {
    var name: String { "Create convex hull around points" }
    var category: String { "Analysis" }
    var description: String { "Create a convex hull for a given set of points. The convex hull is a polygon with shortest perimeter that encloses a set of points." }
    var snippets: [String] { ["CreateConvexHullAroundPointsView.swift"] }
    var tags: Set<String> { ["convex hull", "geometry", "spatial analysis"] }
    
    func makeBody() -> AnyView { .init(CreateConvexHullAroundPointsView()) }
}

struct CreateDynamicBasemapGallery: Sample {
    var name: String { "Create dynamic basemap gallery" }
    var category: String { "Maps" }
    var description: String { "Implement a basemap gallery that automatically retrieves the latest customization options from the basemap styles service." }
    var snippets: [String] { ["CreateDynamicBasemapGalleryView.swift", "CreateDynamicBasemapGalleryView.Views.swift"] }
    var tags: Set<String> { ["basemap", "languages", "service", "style"] }
    
    func makeBody() -> AnyView { .init(CreateDynamicBasemapGalleryView()) }
}

struct CreateLoadReport: Sample {
    var name: String { "Create load report" }
    var category: String { "Utility Networks" }
    var description: String { "Create a simple electric distribution report that displays the count of customers and total load per phase by tracing downstream from a given point." }
    var snippets: [String] { ["CreateLoadReportView.swift", "CreateLoadReportView.Model.swift", "CreateLoadReportView.Views.swift"] }
    var tags: Set<String> { ["condition barriers", "downstream trace", "network analysis", "subnetwork trace", "trace configuration", "traversability", "upstream trace", "utility network", "validate consistency"] }
    
    func makeBody() -> AnyView { .init(CreateLoadReportView()) }
}

struct CreateMobileGeodatabase: Sample {
    var name: String { "Create mobile geodatabase" }
    var category: String { "Edit and Manage Data" }
    var description: String { "Create and share a mobile geodatabase." }
    var snippets: [String] { ["CreateMobileGeodatabaseView.swift", "CreateMobileGeodatabaseView.Model.swift"] }
    var tags: Set<String> { ["arcgis pro", "database", "feature", "feature table", "geodatabase", "mobile geodatabase", "sqlite"] }
    
    func makeBody() -> AnyView { .init(CreateMobileGeodatabaseView()) }
}

struct CreatePlanarAndGeodeticBuffers: Sample {
    var name: String { "Create planar and geodetic buffers" }
    var category: String { "Analysis" }
    var description: String { "Create a planar and geodetic buffer around a map point and display the results as a graphic." }
    var snippets: [String] { ["CreatePlanarAndGeodeticBuffersView.swift"] }
    var tags: Set<String> { ["analysis", "buffer", "euclidean", "geodetic", "geometry", "planar"] }
    
    func makeBody() -> AnyView { .init(CreatePlanarAndGeodeticBuffersView()) }
}

struct CreateSymbolStylesFromWebStyles: Sample {
    var name: String { "Create symbol styles from web styles" }
    var category: String { "Visualization" }
    var description: String { "Create symbol styles from a style file hosted on a portal." }
    var snippets: [String] { ["CreateSymbolStylesFromWebStylesView.swift"] }
    var tags: Set<String> { ["renderer", "symbol", "symbology", "web style"] }
    
    func makeBody() -> AnyView { .init(CreateSymbolStylesFromWebStylesView()) }
}

struct CutGeometry: Sample {
    var name: String { "Cut geometry" }
    var category: String { "Edit and Manage Data" }
    var description: String { "Cut a geometry along a polyline." }
    var snippets: [String] { ["CutGeometryView.swift"] }
    var tags: Set<String> { ["cut", "geometry", "split"] }
    
    func makeBody() -> AnyView { .init(CutGeometryView()) }
}

struct DensifyAndGeneralizeGeometry: Sample {
    var name: String { "Densify and generalize geometry" }
    var category: String { "Edit and Manage Data" }
    var description: String { "A multipart geometry can be densified by adding interpolated points at regular intervals. Generalizing multipart geometry simplifies it while preserving its general shape. Densifying a multipart geometry adds more vertices at regular intervals." }
    var snippets: [String] { ["DensifyAndGeneralizeGeometryView.swift", "DensifyAndGeneralizeGeometryView.SettingsView.swift"] }
    var tags: Set<String> { ["densify", "generalize", "simplify"] }
    
    func makeBody() -> AnyView { .init(DensifyAndGeneralizeGeometryView()) }
}

struct DisplayAnnotation: Sample {
    var name: String { "Display annotation" }
    var category: String { "Layers" }
    var description: String { "Display annotation from a feature service URL." }
    var snippets: [String] { ["DisplayAnnotationView.swift"] }
    var tags: Set<String> { ["annotation", "cartography", "labels", "placement", "reference scale", "text", "utility"] }
    
    func makeBody() -> AnyView { .init(DisplayAnnotationView()) }
}

struct DisplayClusters: Sample {
    var name: String { "Display clusters" }
    var category: String { "Layers" }
    var description: String { "Display a web map with a point feature layer that has feature reduction enabled to aggregate points into clusters." }
    var snippets: [String] { ["DisplayClustersView.swift"] }
    var tags: Set<String> { ["aggregate", "bin", "cluster", "group", "merge", "normalize", "reduce", "summarize"] }
    
    func makeBody() -> AnyView { .init(DisplayClustersView()) }
}

struct DisplayContentOfUtilityNetworkContainer: Sample {
    var name: String { "Display content of utility network container" }
    var category: String { "Utility Networks" }
    var description: String { "A utility network container allows a dense collection of features to be represented by a single feature, which can be used to reduce map clutter." }
    var snippets: [String] { ["DisplayContentOfUtilityNetworkContainerView.swift", "DisplayContentOfUtilityNetworkContainerView.Model.swift"] }
    var tags: Set<String> { ["associations", "connectivity association", "containment association", "structural attachment associations", "utility network"] }
    
    func makeBody() -> AnyView { .init(DisplayContentOfUtilityNetworkContainerView()) }
}

struct DisplayDimensions: Sample {
    var name: String { "Display dimensions" }
    var category: String { "Layers" }
    var description: String { "Display dimension features from a mobile map package." }
    var snippets: [String] { ["DisplayDimensionsView.swift"] }
    var tags: Set<String> { ["definition expression", "dimension", "distance", "layer", "length", "mmpk", "mobile map package", "utility"] }
    var hasDependencies: Bool { true }

    func makeBody() -> AnyView { .init(DisplayDimensionsView()) }
}

struct DisplayMap: Sample {
    var name: String { "Display map" }
    var category: String { "Maps" }
    var description: String { "Display a map with an imagery basemap." }
    var snippets: [String] { ["DisplayMapView.swift"] }
    var tags: Set<String> { ["basemap style", "map"] }
    
    func makeBody() -> AnyView { .init(DisplayMapView()) }
}

struct DisplayMapFromMobileMapPackage: Sample {
    var name: String { "Display map from mobile map package" }
    var category: String { "Edit and Manage Data" }
    var description: String { "Display a map from a mobile map package." }
    var snippets: [String] { ["DisplayMapFromMobileMapPackageView.swift"] }
    var tags: Set<String> { ["mmpk", "mobile map package", "offline"] }
    var hasDependencies: Bool { true }

    func makeBody() -> AnyView { .init(DisplayMapFromMobileMapPackageView()) }
}

struct DisplayMapFromPortalItem: Sample {
    var name: String { "Display map from portal item" }
    var category: String { "Maps" }
    var description: String { "Display a web map." }
    var snippets: [String] { ["DisplayMapFromPortalItemView.swift"] }
    var tags: Set<String> { ["portal item", "web map"] }
    
    func makeBody() -> AnyView { .init(DisplayMapFromPortalItemView()) }
}

struct DisplayOverviewMap: Sample {
    var name: String { "Display overview map" }
    var category: String { "Maps" }
    var description: String { "Include an overview or inset map as an additional map view to show the wider context of the primary view." }
    var snippets: [String] { ["DisplayOverviewMapView.swift"] }
    var tags: Set<String> { ["context", "inset", "map", "minimap", "overview", "preview", "small scale", "toolkit", "view"] }
    
    func makeBody() -> AnyView { .init(DisplayOverviewMapView()) }
}

struct DisplayScene: Sample {
    var name: String { "Display scene" }
    var category: String { "Scenes" }
    var description: String { "Display a scene with a terrain surface and some imagery." }
    var snippets: [String] { ["DisplaySceneView.swift"] }
    var tags: Set<String> { ["3D", "basemap", "elevation", "scene", "surface"] }
    
    func makeBody() -> AnyView { .init(DisplaySceneView()) }
}

struct DisplaySceneFromMobileScenePackage: Sample {
    var name: String { "Display scene from mobile scene package" }
    var category: String { "Scenes" }
    var description: String { "Opens and displays a scene from a Mobile Scene Package (.mspk)." }
    var snippets: [String] { ["DisplaySceneFromMobileScenePackageView.swift"] }
    var tags: Set<String> { ["offline", "scene"] }
    var hasDependencies: Bool { true }

    func makeBody() -> AnyView { .init(DisplaySceneFromMobileScenePackageView()) }
}

struct DisplayWebSceneFromPortalItem: Sample {
    var name: String { "Display web scene from portal item" }
    var category: String { "Scenes" }
    var description: String { "Open a web scene from a portal item." }
    var snippets: [String] { ["DisplayWebSceneFromPortalItemView.swift"] }
    var tags: Set<String> { ["portal", "scene", "web scene"] }
    
    func makeBody() -> AnyView { .init(DisplayWebSceneFromPortalItemView()) }
}

struct DownloadPreplannedMapArea: Sample {
    var name: String { "Download preplanned map area" }
    var category: String { "Edit and Manage Data" }
    var description: String { "Take a map offline using a preplanned map area." }
    var snippets: [String] { ["DownloadPreplannedMapAreaView.swift", "DownloadPreplannedMapAreaView.MapPicker.swift", "DownloadPreplannedMapAreaView.Model.swift"] }
    var tags: Set<String> { ["map area", "offline", "pre-planned", "preplanned"] }
    
    func makeBody() -> AnyView { .init(DownloadPreplannedMapAreaView()) }
}

struct DownloadVectorTilesToLocalCache: Sample {
    var name: String { "Download vector tiles to local cache" }
    var category: String { "Edit and Manage Data" }
    var description: String { "Download tiles from an online vector tile service." }
    var snippets: [String] { ["DownloadVectorTilesToLocalCacheView.swift"] }
    var tags: Set<String> { ["cache", "download", "offline", "vector"] }
    
    func makeBody() -> AnyView { .init(DownloadVectorTilesToLocalCacheView()) }
}

struct EditAndSyncFeaturesWithFeatureService: Sample {
    var name: String { "Edit and sync features with feature service" }
    var category: String { "Edit and Manage Data" }
    var description: String { "Synchronize offline edits with a feature service." }
    var snippets: [String] { ["EditAndSyncFeaturesWithFeatureServiceView.swift", "EditAndSyncFeaturesWithFeatureServiceView.Model.swift"] }
    var tags: Set<String> { ["feature service", "geodatabase", "offline", "synchronize"] }
    var hasDependencies: Bool { true }

    func makeBody() -> AnyView { .init(EditAndSyncFeaturesWithFeatureServiceView()) }
}

struct EditFeatureAttachments: Sample {
    var name: String { "Edit feature attachments" }
    var category: String { "Edit and Manage Data" }
    var description: String { "Add, delete, and download attachments for features from a service." }
    var snippets: [String] { ["EditFeatureAttachmentsView.swift", "EditFeatureAttachmentsView.Model.swift"] }
    var tags: Set<String> { ["Edit and manage data", "JPEG", "PDF", "PNG", "TXT", "image", "picture"] }
    
    func makeBody() -> AnyView { .init(EditFeatureAttachmentsView()) }
}

struct EditFeaturesWithFeatureLinkedAnnotation: Sample {
    var name: String { "Edit features with feature-linked annotation" }
    var category: String { "Edit and Manage Data" }
    var description: String { "Edit feature attributes which are linked to annotations through an expression." }
    var snippets: [String] { ["EditFeaturesWithFeatureLinkedAnnotationView.swift", "EditFeaturesWithFeatureLinkedAnnotationView.Model.swift"] }
    var tags: Set<String> { ["annotation", "attributes", "feature-linked annotation", "features", "fields"] }
    var hasDependencies: Bool { true }

    func makeBody() -> AnyView { .init(EditFeaturesWithFeatureLinkedAnnotationView()) }
}

struct EditGeodatabaseWithTransactions: Sample {
    var name: String { "Edit geodatabase with transactions" }
    var category: String { "Edit and Manage Data" }
    var description: String { "Use transactions to manage how changes are committed to a geodatabase." }
    var snippets: [String] { ["EditGeodatabaseWithTransactionsView.swift", "EditGeodatabaseWithTransactionsView.Model.swift"] }
    var tags: Set<String> { ["commit", "database", "geodatabase", "geometry editor", "transact", "transactions"] }
    
    func makeBody() -> AnyView { .init(EditGeodatabaseWithTransactionsView()) }
}

struct EditWithBranchVersioning: Sample {
    var name: String { "Edit with branch versioning" }
    var category: String { "Edit and Manage Data" }
    var description: String { "Create, query, and edit a specific server version using a service geodatabase." }
    var snippets: [String] { ["EditWithBranchVersioningView.swift", "EditWithBranchVersioningView.Model.swift", "EditWithBranchVersioningView.Views.swift"] }
    var tags: Set<String> { ["branch versioning", "edit", "version control", "version management server"] }
    
    func makeBody() -> AnyView { .init(EditWithBranchVersioningView()) }
}

struct FilterByDefinitionExpressionOrDisplayFilter: Sample {
    var name: String { "Filter by definition expression or display filter" }
    var category: String { "Search and Query" }
    var description: String { "Filter features displayed on a map using a definition expression or a display filter." }
    var snippets: [String] { ["FilterByDefinitionExpressionOrDisplayFilterView.swift"] }
    var tags: Set<String> { ["SQL", "definition expression", "display filter", "filter", "limit data", "query", "restrict data", "where clause"] }
    
    func makeBody() -> AnyView { .init(FilterByDefinitionExpressionOrDisplayFilterView()) }
}

struct FilterFeaturesInScene: Sample {
    var name: String { "Filter features in scene" }
    var category: String { "Scenes" }
    var description: String { "Filter 3D scene features out of a given geometry with a polygon filter." }
    var snippets: [String] { ["FilterFeaturesInSceneView.swift"] }
    var tags: Set<String> { ["3D", "OSM", "buildings", "disjoint", "exclude", "extent", "filter", "hide", "polygon"] }
    
    func makeBody() -> AnyView { .init(FilterFeaturesInSceneView()) }
}

struct FindAddressWithReverseGeocode: Sample {
    var name: String { "Find address with reverse geocode" }
    var category: String { "Search and Query" }
    var description: String { "Use an online service to find the address for a tapped point." }
    var snippets: [String] { ["FindAddressWithReverseGeocodeView.swift"] }
    var tags: Set<String> { ["address", "geocode", "locate", "reverse geocode", "search"] }
    
    func makeBody() -> AnyView { .init(FindAddressWithReverseGeocodeView()) }
}

struct FindClosestFacilityFromPoint: Sample {
    var name: String { "Find closest facility from point" }
    var category: String { "Routing and Logistics" }
    var description: String { "Find routes from several locations to the respective closest facility." }
    var snippets: [String] { ["FindClosestFacilityFromPointView.swift"] }
    var tags: Set<String> { ["incident", "network analysis", "route", "search"] }
    
    func makeBody() -> AnyView { .init(FindClosestFacilityFromPointView()) }
}

struct FindClosestFacilityToMultiplePoints: Sample {
    var name: String { "Find closest facility to multiple points" }
    var category: String { "Routing and Logistics" }
    var description: String { "Find a route to the closest facility from a location." }
    var snippets: [String] { ["FindClosestFacilityToMultiplePointsView.swift"] }
    var tags: Set<String> { ["incident", "network analysis", "route", "search"] }
    
    func makeBody() -> AnyView { .init(FindClosestFacilityToMultiplePointsView()) }
}

struct FindNearestVertex: Sample {
    var name: String { "Find nearest vertex" }
    var category: String { "Analysis" }
    var description: String { "Find the closest vertex and coordinate of a geometry to a point." }
    var snippets: [String] { ["FindNearestVertexView.swift"] }
    var tags: Set<String> { ["analysis", "coordinate", "geometry", "nearest", "proximity", "vertex"] }
    
    func makeBody() -> AnyView { .init(FindNearestVertexView()) }
}

struct FindRoute: Sample {
    var name: String { "Find route" }
    var category: String { "Routing and Logistics" }
    var description: String { "Display directions for a route between two points." }
    var snippets: [String] { ["FindRouteView.swift"] }
    var tags: Set<String> { ["directions", "driving", "navigation", "network", "network analysis", "route", "routing", "shortest path", "turn-by-turn"] }
    
    func makeBody() -> AnyView { .init(FindRouteView()) }
}

struct FindRouteAroundBarriers: Sample {
    var name: String { "Find route around barriers" }
    var category: String { "Routing and Logistics" }
    var description: String { "Find a route that reaches all stops without crossing any barriers." }
    var snippets: [String] { ["FindRouteAroundBarriersView.swift", "FindRouteAroundBarriersView.Model.swift", "FindRouteAroundBarriersView.Views.swift"] }
    var tags: Set<String> { ["barriers", "best sequence", "directions", "maneuver", "network analysis", "routing", "sequence", "stop order", "stops"] }
    
    func makeBody() -> AnyView { .init(FindRouteAroundBarriersView()) }
}

struct FindRouteInMobileMapPackage: Sample {
    var name: String { "Find route in mobile map package" }
    var category: String { "Maps" }
    var description: String { "Display maps and use locators to enable search and routing offline using a Mobile Map Package." }
    var snippets: [String] { ["FindRouteInMobileMapPackageView.swift", "FindRouteInMobileMapPackageView.Models.swift", "FindRouteInMobileMapPackageView.MobileMapView.swift"] }
    var tags: Set<String> { ["disconnected", "field mobility", "geocode", "network", "network analysis", "offline", "routing", "search", "transportation"] }
    var hasDependencies: Bool { true }

    func makeBody() -> AnyView { .init(FindRouteInMobileMapPackageView()) }
}

struct FindRouteInTransportNetwork: Sample {
    var name: String { "Find route in transport network" }
    var category: String { "Routing and Logistics" }
    var description: String { "Solve a route on-the-fly using offline data." }
    var snippets: [String] { ["FindRouteInTransportNetworkView.swift", "FindRouteInTransportNetworkView.Model.swift"] }
    var tags: Set<String> { ["connectivity", "disconnected", "fastest", "locator", "navigation", "network analysis", "offline", "routing", "shortest", "turn-by-turn"] }
    var hasDependencies: Bool { true }

    func makeBody() -> AnyView { .init(FindRouteInTransportNetworkView()) }
}

struct GenerateOfflineMap: Sample {
    var name: String { "Generate offline map" }
    var category: String { "Edit and Manage Data" }
    var description: String { "Take a web map offline." }
    var snippets: [String] { ["GenerateOfflineMapView.swift"] }
    var tags: Set<String> { ["download", "offline", "save", "web map"] }
    
    func makeBody() -> AnyView { .init(GenerateOfflineMapView()) }
}

struct GenerateOfflineMapWithCustomParameters: Sample {
    var name: String { "Generate offline map with custom parameters" }
    var category: String { "Edit and Manage Data" }
    var description: String { "Take a web map offline with additional options for each layer." }
    var snippets: [String] { ["GenerateOfflineMapWithCustomParametersView.swift", "GenerateOfflineMapWithCustomParametersView.Model.swift", "GenerateOfflineMapWithCustomParametersView.CustomParameters.swift"] }
    var tags: Set<String> { ["LOD", "adjust", "download", "extent", "filter", "offline", "override", "parameters", "reduce", "scale range", "setting"] }
    
    func makeBody() -> AnyView { .init(GenerateOfflineMapWithCustomParametersView()) }
}

struct GenerateOfflineMapWithLocalBasemap: Sample {
    var name: String { "Generate offline map with local basemap" }
    var category: String { "Edit and Manage Data" }
    var description: String { "Use the `OfflineMapTask` to take a web map offline, but instead of downloading an online basemap, use one which is already on the device." }
    var snippets: [String] { ["GenerateOfflineMapWithLocalBasemapView.swift"] }
    var tags: Set<String> { ["basemap", "download", "local", "offline", "save", "web map"] }
    var hasDependencies: Bool { true }

    func makeBody() -> AnyView { .init(GenerateOfflineMapWithLocalBasemapView()) }
}

struct GeocodeOffline: Sample {
    var name: String { "Geocode offline" }
    var category: String { "Search and Query" }
    var description: String { "Geocode addresses to locations and reverse geocode locations to addresses offline." }
    var snippets: [String] { ["GeocodeOfflineView.swift", "GeocodeOfflineView.Model.swift"] }
    var tags: Set<String> { ["geocode", "geocoder", "locator", "offline", "package", "query", "search"] }
    var hasDependencies: Bool { true }

    func makeBody() -> AnyView { .init(GeocodeOfflineView()) }
}

struct GetElevationAtPointOnSurface: Sample {
    var name: String { "Get elevation at point on surface" }
    var category: String { "Scenes" }
    var description: String { "Get the elevation for a given point on a surface in a scene." }
    var snippets: [String] { ["GetElevationAtPointOnSurfaceView.swift"] }
    var tags: Set<String> { ["elevation", "point", "surface"] }
    
    func makeBody() -> AnyView { .init(GetElevationAtPointOnSurfaceView()) }
}

struct GroupLayersTogether: Sample {
    var name: String { "Group layers together" }
    var category: String { "Layers" }
    var description: String { "Group a collection of layers together and toggle their visibility as a group." }
    var snippets: [String] { ["GroupLayersTogetherView.swift", "GroupLayersTogetherView.GroupLayerListView.swift"] }
    var tags: Set<String> { ["group layer", "layers"] }
    
    func makeBody() -> AnyView { .init(GroupLayersTogetherView()) }
}

struct IdentifyKMLFeatures: Sample {
    var name: String { "Identify KML features" }
    var category: String { "Layers" }
    var description: String { "Show a callout with formatted content for a KML feature." }
    var snippets: [String] { ["IdentifyKMLFeaturesView.swift"] }
    var tags: Set<String> { ["KML", "KMZ", "Keyhole", "NOAA", "NWS", "OGC", "weather"] }
    
    func makeBody() -> AnyView { .init(IdentifyKMLFeaturesView()) }
}

struct IdentifyFeaturesInWMSLayer: Sample {
    var name: String { "Identify features in WMS layer" }
    var category: String { "Layers" }
    var description: String { "Identify features in a WMS layer and display the associated popup content." }
    var snippets: [String] { ["IdentifyFeaturesInWMSLayerView.swift"] }
    var tags: Set<String> { ["OGC", "WMS", "web map service"] }
    
    func makeBody() -> AnyView { .init(IdentifyFeaturesInWMSLayerView()) }
}

struct IdentifyGraphics: Sample {
    var name: String { "Identify graphics" }
    var category: String { "Visualization" }
    var description: String { "Display an alert message when a graphic is tapped." }
    var snippets: [String] { ["IdentifyGraphicsView.swift"] }
    var tags: Set<String> { ["graphics", "identify"] }
    
    func makeBody() -> AnyView { .init(IdentifyGraphicsView()) }
}

struct IdentifyLayerFeatures: Sample {
    var name: String { "Identify layer features" }
    var category: String { "Search and Query" }
    var description: String { "Identify features in all layers in a map." }
    var snippets: [String] { ["IdentifyLayerFeaturesView.swift"] }
    var tags: Set<String> { ["identify", "recursion", "recursive", "sublayers"] }
    
    func makeBody() -> AnyView { .init(IdentifyLayerFeaturesView()) }
}

struct IdentifyRasterCell: Sample {
    var name: String { "Identify raster cell" }
    var category: String { "Layers" }
    var description: String { "Get the cell value of a local raster at the tapped location and display the result in a callout." }
    var snippets: [String] { ["IdentifyRasterCellView.swift"] }
    var tags: Set<String> { ["NDVI", "band", "cell", "cell value", "continuous", "discrete", "identify", "pixel", "pixel value", "raster"] }
    var hasDependencies: Bool { true }

    func makeBody() -> AnyView { .init(IdentifyRasterCellView()) }
}

struct ListSpatialReferenceTransformations: Sample {
    var name: String { "List spatial reference transformations" }
    var category: String { "Edit and Manage Data" }
    var description: String { "Get a list of suitable transformations for projecting a geometry between two spatial references with different horizontal datums." }
    var snippets: [String] { ["ListSpatialReferenceTransformationsView.swift", "ListSpatialReferenceTransformationsView.Model.swift"] }
    var tags: Set<String> { ["datum", "geodesy", "projection", "spatial reference", "transformation"] }
    
    func makeBody() -> AnyView { .init(ListSpatialReferenceTransformationsView()) }
}

struct ManageBookmarks: Sample {
    var name: String { "Manage bookmarks" }
    var category: String { "Maps" }
    var description: String { "Access and create bookmarks on a map." }
    var snippets: [String] { ["ManageBookmarksView.swift"] }
    var tags: Set<String> { ["bookmark", "extent", "location", "zoom"] }
    
    func makeBody() -> AnyView { .init(ManageBookmarksView()) }
}

struct ManageOperationalLayers: Sample {
    var name: String { "Manage operational layers" }
    var category: String { "Maps" }
    var description: String { "Add, remove, and reorder operational layers in a map." }
    var snippets: [String] { ["ManageOperationalLayersView.swift"] }
    var tags: Set<String> { ["add", "delete", "layer", "map", "remove"] }
    
    func makeBody() -> AnyView { .init(ManageOperationalLayersView()) }
}

struct MatchViewpointOfGeoViews: Sample {
    var name: String { "Match viewpoint of geo views" }
    var category: String { "Scenes" }
    var description: String { "Keep the viewpoints of two views, for example, a map view and a scene view, synchronized with each other." }
    var snippets: [String] { ["MatchViewpointOfGeoViewsView.swift"] }
    var tags: Set<String> { ["3D", "automatic refresh", "event", "event handler", "events", "extent", "interaction", "interactions", "pan", "zoom"] }
    
    func makeBody() -> AnyView { .init(MatchViewpointOfGeoViewsView()) }
}

struct MeasureDistanceInScene: Sample {
    var name: String { "Measure distance in scene" }
    var category: String { "Analysis" }
    var description: String { "Measure distances between two points in 3D." }
    var snippets: [String] { ["MeasureDistanceInSceneView.swift"] }
    var tags: Set<String> { ["3D", "analysis", "distance", "measure"] }
    
    func makeBody() -> AnyView { .init(MeasureDistanceInSceneView()) }
}

struct MonitorChangesToDrawStatus: Sample {
    var name: String { "Monitor changes to draw status" }
    var category: String { "Maps" }
    var description: String { "Get the draw status of your map view or scene view to know when all layers in the map or scene have finished drawing." }
    var snippets: [String] { ["MonitorChangesToDrawStatusView.swift"] }
    var tags: Set<String> { ["draw", "loading", "map", "render"] }
    
    func makeBody() -> AnyView { .init(MonitorChangesToDrawStatusView()) }
}

struct MonitorChangesToLayerViewState: Sample {
    var name: String { "Monitor changes to layer view state" }
    var category: String { "Maps" }
    var description: String { "Determine whether a layer is currently visible." }
    var snippets: [String] { ["MonitorChangesToLayerViewStateView.swift"] }
    var tags: Set<String> { ["layer", "load", "map", "status", "view", "visibility"] }
    
    func makeBody() -> AnyView { .init(MonitorChangesToLayerViewStateView()) }
}

struct MonitorChangesToMapLoadStatus: Sample {
    var name: String { "Monitor changes to map load status" }
    var category: String { "Maps" }
    var description: String { "Determine the map's load status which can be: `notLoaded`, `failed`, `loading`, or `loaded`." }
    var snippets: [String] { ["MonitorChangesToMapLoadStatusView.swift"] }
    var tags: Set<String> { ["load status", "loadable pattern", "map"] }
    
    func makeBody() -> AnyView { .init(MonitorChangesToMapLoadStatusView()) }
}

struct NavigateRoute: Sample {
    var name: String { "Navigate route" }
    var category: String { "Routing and Logistics" }
    var description: String { "Use a routing service to navigate between two points." }
    var snippets: [String] { ["NavigateRouteView.swift"] }
    var tags: Set<String> { ["directions", "maneuver", "navigation", "route", "turn-by-turn", "voice"] }
    
    func makeBody() -> AnyView { .init(NavigateRouteView()) }
}

struct NavigateRouteWithRerouting: Sample {
    var name: String { "Navigate route with rerouting" }
    var category: String { "Routing and Logistics" }
    var description: String { "Navigate between two points and dynamically recalculate an alternate route when the original route is unavailable." }
    var snippets: [String] { ["NavigateRouteWithReroutingView.swift", "NavigateRouteWithReroutingView.Model.swift"] }
    var tags: Set<String> { ["directions", "maneuver", "navigation", "route", "turn-by-turn", "voice"] }
    var hasDependencies: Bool { true }

    func makeBody() -> AnyView { .init(NavigateRouteWithReroutingView()) }
}

struct OrbitCameraAroundObject: Sample {
    var name: String { "Orbit camera around object" }
    var category: String { "Scenes" }
    var description: String { "Fix the camera to point at and rotate around a target object." }
    var snippets: [String] { ["OrbitCameraAroundObjectView.swift", "OrbitCameraAroundObjectView.Model.swift"] }
    var tags: Set<String> { ["3D", "camera", "object", "orbit", "rotate", "scene"] }
    var hasDependencies: Bool { true }

    func makeBody() -> AnyView { .init(OrbitCameraAroundObjectView()) }
}

struct PlayKMLTour: Sample {
    var name: String { "Play KML tour" }
    var category: String { "Visualization" }
    var description: String { "Play tours in KML files." }
    var snippets: [String] { ["PlayKMLTourView.swift"] }
    var tags: Set<String> { ["KML", "animation", "interactive", "narration", "pause", "play", "story", "tour"] }
    
    func makeBody() -> AnyView { .init(PlayKMLTourView()) }
}

struct ProjectGeometry: Sample {
    var name: String { "Project geometry" }
    var category: String { "Edit and Manage Data" }
    var description: String { "Project a point to another spatial reference." }
    var snippets: [String] { ["ProjectGeometryView.swift"] }
    var tags: Set<String> { ["WGS 84", "Web Mercator", "coordinate system", "coordinates", "latitude", "longitude", "projected", "projection", "spatial reference"] }
    
    func makeBody() -> AnyView { .init(ProjectGeometryView()) }
}

struct QueryFeatureTable: Sample {
    var name: String { "Query feature table" }
    var category: String { "Search and Query" }
    var description: String { "Find features in a feature table which match an SQL query." }
    var snippets: [String] { ["QueryFeatureTableView.swift"] }
    var tags: Set<String> { ["query", "search"] }
    
    func makeBody() -> AnyView { .init(QueryFeatureTableView()) }
}

struct QueryFeaturesWithArcadeExpression: Sample {
    var name: String { "Query features with Arcade expression" }
    var category: String { "Search and Query" }
    var description: String { "Query features on a map using an Arcade expression." }
    var snippets: [String] { ["QueryFeaturesWithArcadeExpressionView.swift"] }
    var tags: Set<String> { ["Arcade evaluator", "Arcade expression", "identify layers", "portal", "portal item", "query"] }
    
    func makeBody() -> AnyView { .init(QueryFeaturesWithArcadeExpressionView()) }
}

struct QueryRelatedFeatures: Sample {
    var name: String { "Query related features" }
    var category: String { "Search and Query" }
    var description: String { "List features related to the selected feature." }
    var snippets: [String] { ["QueryRelatedFeaturesView.swift"] }
    var tags: Set<String> { ["features", "identify", "query", "related", "relationship", "search"] }
    
    func makeBody() -> AnyView { .init(QueryRelatedFeaturesView()) }
}

struct RenderMultilayerSymbols: Sample {
    var name: String { "Render multilayer symbols" }
    var category: String { "Visualization" }
    var description: String { "Show different kinds of multilayer symbols on a map similar to some pre-defined 2D simple symbol styles." }
    var snippets: [String] { ["RenderMultilayerSymbolsView.swift"] }
    var tags: Set<String> { ["graphic", "marker", "multilayer", "picture", "symbol"] }
    
    func makeBody() -> AnyView { .init(RenderMultilayerSymbolsView()) }
}

struct RunValveIsolationTrace: Sample {
    var name: String { "Run valve isolation trace" }
    var category: String { "Utility Networks" }
    var description: String { "Run a filtered trace to locate operable features that will isolate an area from the flow of network resources." }
    var snippets: [String] { ["RunValveIsolationTraceView.swift", "RunValveIsolationTraceView.Model.swift"] }
    var tags: Set<String> { ["category comparison", "condition barriers", "filter barriers", "isolated features", "network analysis", "subnetwork trace", "trace configuration", "trace filter", "utility network"] }
    
    func makeBody() -> AnyView { .init(RunValveIsolationTraceView()) }
}

struct SearchForWebMap: Sample {
    var name: String { "Search for web map" }
    var category: String { "Cloud and Portal" }
    var description: String { "Find web map portal items by using a search term." }
    var snippets: [String] { ["SearchForWebMapView.swift", "SearchForWebMapView.Model.swift", "SearchForWebMapView.Views.swift"] }
    var tags: Set<String> { ["keyword", "query", "search", "web map"] }
    
    func makeBody() -> AnyView { .init(SearchForWebMapView()) }
}

struct SearchWithGeocode: Sample {
    var name: String { "Search with geocode" }
    var category: String { "Search and Query" }
    var description: String { "Find the location for an address, or places of interest near a location or within a specific area." }
    var snippets: [String] { ["SearchWithGeocodeView.swift"] }
    var tags: Set<String> { ["POI", "address", "businesses", "geocode", "locations", "locator", "places of interest", "point of interest", "search", "suggestions", "toolkit"] }
    
    func makeBody() -> AnyView { .init(SearchWithGeocodeView()) }
}

struct SelectFeaturesInFeatureLayer: Sample {
    var name: String { "Select features in feature layer" }
    var category: String { "Search and Query" }
    var description: String { "Select features in a feature layer." }
    var snippets: [String] { ["SelectFeaturesInFeatureLayerView.swift"] }
    var tags: Set<String> { ["features", "layers", "select", "selection", "tolerance"] }
    
    func makeBody() -> AnyView { .init(SelectFeaturesInFeatureLayerView()) }
}

struct SelectFeaturesInSceneLayer: Sample {
    var name: String { "Select features in scene layer" }
    var category: String { "Scenes" }
    var description: String { "Identify features in a scene to select." }
    var snippets: [String] { ["SelectFeaturesInSceneLayerView.swift"] }
    var tags: Set<String> { ["3D", "buildings", "identify", "model", "query", "search", "select"] }
    
    func makeBody() -> AnyView { .init(SelectFeaturesInSceneLayerView()) }
}

struct SetBasemap: Sample {
    var name: String { "Set basemap" }
    var category: String { "Maps" }
    var description: String { "Set a map's basemap. A basemap is beneath all layers on a map and is used to provide visual reference for the operational layers." }
    var snippets: [String] { ["SetBasemapView.swift"] }
    var tags: Set<String> { ["basemap", "map"] }
    
    func makeBody() -> AnyView { .init(SetBasemapView()) }
}

struct SetFeatureRequestMode: Sample {
    var name: String { "Set feature request mode" }
    var category: String { "Search and Query" }
    var description: String { "Use different feature request modes to populate the map from a service feature table." }
    var snippets: [String] { ["SetFeatureRequestModeView.swift"] }
    var tags: Set<String> { ["cache", "data", "feature", "feature request mode", "performance"] }
    
    func makeBody() -> AnyView { .init(SetFeatureRequestModeView()) }
}

struct SetMaxExtent: Sample {
    var name: String { "Set max extent" }
    var category: String { "Maps" }
    var description: String { "Limit the view of a map to a particular area." }
    var snippets: [String] { ["SetMaxExtentView.swift"] }
    var tags: Set<String> { ["extent", "limit panning", "map", "mapview", "max extent", "zoom"] }
    
    func makeBody() -> AnyView { .init(SetMaxExtentView()) }
}

struct SetMinAndMaxScale: Sample {
    var name: String { "Set min and max scale" }
    var category: String { "Maps" }
    var description: String { "Restrict zooming between specific scale ranges." }
    var snippets: [String] { ["SetMinAndMaxScaleView.swift"] }
    var tags: Set<String> { ["area of interest", "level of detail", "maximum", "minimum", "scale", "viewpoint"] }
    
    func makeBody() -> AnyView { .init(SetMinAndMaxScaleView()) }
}

struct SetReferenceScale: Sample {
    var name: String { "Set reference scale" }
    var category: String { "Maps" }
    var description: String { "Set the map's reference scale and which feature layers should honor the reference scale." }
    var snippets: [String] { ["SetReferenceScaleView.swift"] }
    var tags: Set<String> { ["map", "reference scale", "scene"] }
    
    func makeBody() -> AnyView { .init(SetReferenceScaleView()) }
}

struct SetSurfaceNavigationConstraint: Sample {
    var name: String { "Set surface navigation constraint" }
    var category: String { "Scenes" }
    var description: String { "See through terrain in a scene and move the camera underground." }
    var snippets: [String] { ["SetSurfaceNavigationConstraintView.swift"] }
    var tags: Set<String> { ["3D", "subsurface", "underground", "utilities"] }
    
    func makeBody() -> AnyView { .init(SetSurfaceNavigationConstraintView()) }
}

struct SetSurfacePlacementMode: Sample {
    var name: String { "Set surface placement mode" }
    var category: String { "Scenes" }
    var description: String { "Position graphics relative to a surface using different surface placement modes." }
    var snippets: [String] { ["SetSurfacePlacementModeView.swift"] }
    var tags: Set<String> { ["3D", "absolute", "altitude", "draped", "elevation", "floating", "relative", "scenes", "sea level", "surface placement"] }
    
    func makeBody() -> AnyView { .init(SetSurfacePlacementModeView()) }
}

struct SetUpLocationDrivenGeotriggers: Sample {
    var name: String { "Set up location-driven geotriggers" }
    var category: String { "Maps" }
    var description: String { "Create a notification every time a given location data source has entered and/or exited a set of features or graphics." }
    var snippets: [String] { ["SetUpLocationDrivenGeotriggersView.swift", "SetUpLocationDrivenGeotriggersView.Model.swift"] }
    var tags: Set<String> { ["alert", "arcade", "fence", "geofence", "geotrigger", "location", "navigation", "notification", "notify", "routing", "trigger"] }
    
    func makeBody() -> AnyView { .init(SetUpLocationDrivenGeotriggersView()) }
}

struct SetViewpointRotation: Sample {
    var name: String { "Set viewpoint rotation" }
    var category: String { "Maps" }
    var description: String { "Rotate a map." }
    var snippets: [String] { ["SetViewpointRotationView.swift"] }
    var tags: Set<String> { ["rotate", "rotation", "toolkit", "viewpoint"] }
    
    func makeBody() -> AnyView { .init(SetViewpointRotationView()) }
}

struct SetVisibilityOfSubtypeSublayer: Sample {
    var name: String { "Set visibility of subtype sublayer" }
    var category: String { "Layers" }
    var description: String { "Display a composite layer of all the subtype values in a feature class." }
    var snippets: [String] { ["SetVisibilityOfSubtypeSublayerView.swift", "SetVisibilityOfSubtypeSublayerView.Model.swift", "SetVisibilityOfSubtypeSublayerView.Views.swift"] }
    var tags: Set<String> { ["asset group", "feature layer", "labeling", "sublayer", "subtype", "symbology", "utility network", "visible scale range"] }
    
    func makeBody() -> AnyView { .init(SetVisibilityOfSubtypeSublayerView()) }
}

struct ShowCallout: Sample {
    var name: String { "Show callout" }
    var category: String { "Maps" }
    var description: String { "Show a callout with the latitude and longitude of user-tapped points." }
    var snippets: [String] { ["ShowCalloutView.swift"] }
    var tags: Set<String> { ["balloon", "bubble", "callout", "flyout", "flyover", "info window", "popup", "tap"] }
    
    func makeBody() -> AnyView { .init(ShowCalloutView()) }
}

struct ShowCoordinatesInMultipleFormats: Sample {
    var name: String { "Show coordinates in multiple formats" }
    var category: String { "Edit and Manage Data" }
    var description: String { "Format coordinates in a variety of common notations." }
    var snippets: [String] { ["ShowCoordinatesInMultipleFormatsView.swift"] }
    var tags: Set<String> { ["USNG", "UTM", "convert", "coordinate", "decimal degrees", "degree minutes seconds", "format", "latitude", "longitude"] }
    
    func makeBody() -> AnyView { .init(ShowCoordinatesInMultipleFormatsView()) }
}

struct ShowDeviceLocation: Sample {
    var name: String { "Show device location" }
    var category: String { "Maps" }
    var description: String { "Show your current position on the map, as well as switch between different types of auto-pan modes." }
    var snippets: [String] { ["ShowDeviceLocationView.swift"] }
    var tags: Set<String> { ["GPS", "compass", "location", "map", "mobile", "navigation"] }
    
    func makeBody() -> AnyView { .init(ShowDeviceLocationView()) }
}

struct ShowDeviceLocationHistory: Sample {
    var name: String { "Show device location history" }
    var category: String { "Maps" }
    var description: String { "Display your location history on the map." }
    var snippets: [String] { ["ShowDeviceLocationHistoryView.swift"] }
    var tags: Set<String> { ["GPS", "bread crumb", "breadcrumb", "history", "movement", "navigation", "real-time", "trace", "track", "trail"] }
    
    func makeBody() -> AnyView { .init(ShowDeviceLocationHistoryView()) }
}

struct ShowDeviceLocationWithNMEADataSources: Sample {
    var name: String { "Show device location with NMEA data sources" }
    var category: String { "Maps" }
    var description: String { "Parse NMEA sentences and use the results to show device location on the map." }
    var snippets: [String] { ["ShowDeviceLocationWithNMEADataSourcesView.swift", "ShowDeviceLocationWithNMEADataSourcesView.Model.swift", "FileNMEASentenceReader.swift"] }
    var tags: Set<String> { ["Bluetooth", "GNSS", "GPS", "NMEA", "accessory", "history", "navigation", "real-time", "trace"] }
    var hasDependencies: Bool { true }

    func makeBody() -> AnyView { .init(ShowDeviceLocationWithNMEADataSourcesView()) }
}

struct ShowExtrudedFeatures: Sample {
    var name: String { "Show extruded features" }
    var category: String { "Visualization" }
    var description: String { "Extrude features based on their attributes." }
    var snippets: [String] { ["ShowExtrudedFeaturesView.swift"] }
    var tags: Set<String> { ["3D", "extrude", "extrusion", "extrusion expression", "height", "renderer", "scene"] }
    
    func makeBody() -> AnyView { .init(ShowExtrudedFeaturesView()) }
}

struct ShowGrid: Sample {
    var name: String { "Show grid" }
    var category: String { "Maps" }
    var description: String { "Display coordinate system grids including latitude-longitude, MGRS, UTM and USNG on a map view. Also, toggle label visibility and change the color of grid lines and grid labels." }
    var snippets: [String] { ["ShowGridView.swift"] }
    var tags: Set<String> { ["MGRS", "USNG", "UTM", "coordinates", "degrees", "graticule", "grid", "latitude", "longitude", "minutes", "seconds"] }
    
    func makeBody() -> AnyView { .init(ShowGridView()) }
}

struct ShowLabelsOnLayer: Sample {
    var name: String { "Show labels on layer" }
    var category: String { "Visualization" }
    var description: String { "Display custom labels on a feature layer." }
    var snippets: [String] { ["ShowLabelsOnLayerView.swift"] }
    var tags: Set<String> { ["arcade", "attribute", "deconfliction", "label", "labeling", "string", "symbol", "text", "visualization"] }
    
    func makeBody() -> AnyView { .init(ShowLabelsOnLayerView()) }
}

struct ShowLineOfSightBetweenPoints: Sample {
    var name: String { "Show line of sight between points" }
    var category: String { "Analysis" }
    var description: String { "Perform a line of sight analysis between two points in real time." }
    var snippets: [String] { ["ShowLineOfSightBetweenPointsView.swift"] }
    var tags: Set<String> { ["3D", "line of sight", "visibility", "visibility analysis"] }
    
    func makeBody() -> AnyView { .init(ShowLineOfSightBetweenPointsView()) }
}

struct ShowMobileMapPackageExpirationDate: Sample {
    var name: String { "Show mobile map package expiration date" }
    var category: String { "Maps" }
    var description: String { "Access the expiration information of an expired mobile map package." }
    var snippets: [String] { ["ShowMobileMapPackageExpirationDateView.swift"] }
    var tags: Set<String> { ["expiration", "mmpk"] }
    var hasDependencies: Bool { true }

    func makeBody() -> AnyView { .init(ShowMobileMapPackageExpirationDateView()) }
}

struct ShowPopup: Sample {
    var name: String { "Show popup" }
    var category: String { "Edit and Manage Data" }
    var description: String { "Show a predefined popup from a web map." }
    var snippets: [String] { ["ShowPopupView.swift"] }
    var tags: Set<String> { ["feature", "feature layer", "popup", "web map"] }
    
    func makeBody() -> AnyView { .init(ShowPopupView()) }
}

struct ShowRealisticLightAndShadows: Sample {
    var name: String { "Show realistic light and shadows" }
    var category: String { "Scenes" }
    var description: String { "Show realistic lighting and shadows for a given time of day." }
    var snippets: [String] { ["ShowRealisticLightAndShadowsView.swift"] }
    var tags: Set<String> { ["3D", "lighting", "realism", "realistic", "rendering", "shadows", "sun", "time"] }
    
    func makeBody() -> AnyView { .init(ShowRealisticLightAndShadowsView()) }
}

struct ShowResultOfSpatialOperations: Sample {
    var name: String { "Show result of spatial operations" }
    var category: String { "Edit and Manage Data" }
    var description: String { "Find the union, intersection, or difference of two geometries." }
    var snippets: [String] { ["ShowResultOfSpatialOperationsView.swift"] }
    var tags: Set<String> { ["analysis", "combine", "difference", "geometry", "intersection", "merge", "polygon", "union"] }
    
    func makeBody() -> AnyView { .init(ShowResultOfSpatialOperationsView()) }
}

struct ShowResultOfSpatialRelationships: Sample {
    var name: String { "Show result of spatial relationships" }
    var category: String { "Analysis" }
    var description: String { "Determine spatial relationships between two geometries." }
    var snippets: [String] { ["ShowResultOfSpatialRelationshipsView.swift"] }
    var tags: Set<String> { ["geometries", "relationship", "spatial analysis"] }
    
    func makeBody() -> AnyView { .init(ShowResultOfSpatialRelationshipsView()) }
}

struct ShowScaleBar: Sample {
    var name: String { "Show scale bar" }
    var category: String { "Maps" }
    var description: String { "Add a scale bar to visually gauge distances on a map." }
    var snippets: [String] { ["ShowScaleBarView.swift"] }
    var tags: Set<String> { ["map", "measure", "scale", "toolkit"] }
    
    func makeBody() -> AnyView { .init(ShowScaleBarView()) }
}

struct ShowServiceArea: Sample {
    var name: String { "Show service area" }
    var category: String { "Analysis" }
    var description: String { "Find the service area within a network from a given point." }
    var snippets: [String] { ["ShowServiceAreaView.swift"] }
    var tags: Set<String> { ["barriers", "facilities", "impedance", "logistics", "routing"] }
    
    func makeBody() -> AnyView { .init(ShowServiceAreaView()) }
}

struct ShowUtilityAssociations: Sample {
    var name: String { "Show utility associations" }
    var category: String { "Utility Networks" }
    var description: String { "Create graphics for utility associations in a utility network." }
    var snippets: [String] { ["ShowUtilityAssociationsView.swift"] }
    var tags: Set<String> { ["associating", "association", "attachment", "connectivity", "containment", "relationships"] }
    
    func makeBody() -> AnyView { .init(ShowUtilityAssociationsView()) }
}

struct ShowViewshedFromGeoelementInScene: Sample {
    var name: String { "Show viewshed from geoelement in scene" }
    var category: String { "Analysis" }
    var description: String { "Analyze the viewshed for an object (GeoElement) in a scene." }
    var snippets: [String] { ["ShowViewshedFromGeoelementInSceneView.swift"] }
    var tags: Set<String> { ["3D", "analysis", "buildings", "model", "scene", "viewshed", "visibility analysis"] }
    var hasDependencies: Bool { true }

    func makeBody() -> AnyView { .init(ShowViewshedFromGeoelementInSceneView()) }
}

struct ShowViewshedFromPointInScene: Sample {
    var name: String { "Show viewshed from point in scene" }
    var category: String { "Analysis" }
    var description: String { "Perform a viewshed analysis from a defined vantage point." }
    var snippets: [String] { ["ShowViewshedFromPointInSceneView.swift", "ShowViewshedFromPointInSceneView.Model.swift", "ShowViewshedFromPointInSceneView.ViewshedSettingsView.swift"] }
    var tags: Set<String> { ["3D", "frustum", "scene", "viewshed", "visibility analysis"] }
    
    func makeBody() -> AnyView { .init(ShowViewshedFromPointInSceneView()) }
}

struct ShowViewshedFromPointOnMap: Sample {
    var name: String { "Show viewshed from point on map" }
    var category: String { "Analysis" }
    var description: String { "Calculate a viewshed using a geoprocessing service, in this case showing what parts of a landscape are visible from points on mountainous terrain." }
    var snippets: [String] { ["ShowViewshedFromPointOnMapView.swift"] }
    var tags: Set<String> { ["geoprocessing", "heat map", "heatmap", "viewshed"] }
    
    func makeBody() -> AnyView { .init(ShowViewshedFromPointOnMapView()) }
}

struct SnapGeometryEdits: Sample {
    var name: String { "Snap geometry edits" }
    var category: String { "Edit and Manage Data" }
    var description: String { "Use the Geometry Editor to edit a geometry and align it to existing geometries on a map." }
    var snippets: [String] { ["SnapGeometryEditsView.swift", "SnapGeometryEditsView.SnapSettingsView.swift", "SnapGeometryEditsView.GeometryEditorModel.swift", "SnapGeometryEditsView.GeometryEditorMenu.swift"] }
    var tags: Set<String> { ["edit", "feature", "geometry editor", "graphics", "layers", "map", "snapping"] }
    
    func makeBody() -> AnyView { .init(SnapGeometryEditsView()) }
}

struct StyleFeaturesWithCustomDictionary: Sample {
    var name: String { "Style features with custom dictionary" }
    var category: String { "Visualization" }
    var description: String { "Use a custom dictionary style created from a web style or local style file (.stylx) to symbolize features using a variety of attribute values." }
    var snippets: [String] { ["StyleFeaturesWithCustomDictionaryView.swift"] }
    var tags: Set<String> { ["dictionary", "military", "portal", "portal item", "renderer", "style", "stylx", "unique value", "visualization"] }
    var hasDependencies: Bool { true }

    func makeBody() -> AnyView { .init(StyleFeaturesWithCustomDictionaryView()) }
}

struct StyleGraphicsWithRenderer: Sample {
    var name: String { "Style graphics with renderer" }
    var category: String { "Visualization" }
    var description: String { "A renderer allows you to change the style of all graphics in a graphics overlay by referencing a single symbol style. A renderer will only affect graphics that do not specify their own symbol style." }
    var snippets: [String] { ["StyleGraphicsWithRendererView.swift"] }
    var tags: Set<String> { ["arc", "bezier", "curve", "display", "ellipse", "graphics", "marker", "overlay", "renderer", "segment", "symbol", "true curve"] }
    
    func makeBody() -> AnyView { .init(StyleGraphicsWithRendererView()) }
}

struct StyleGraphicsWithSymbols: Sample {
    var name: String { "Style graphics with symbols" }
    var category: String { "Visualization" }
    var description: String { "Use a symbol style to display a graphic on a graphics overlay." }
    var snippets: [String] { ["StyleGraphicsWithSymbolsView.swift"] }
    var tags: Set<String> { ["display", "fill", "graphics", "line", "marker", "overlay", "point", "symbol"] }
    
    func makeBody() -> AnyView { .init(StyleGraphicsWithSymbolsView()) }
}

struct StylePointWithDistanceCompositeSceneSymbol: Sample {
    var name: String { "Style point with distance composite scene symbol" }
    var category: String { "Scenes" }
    var description: String { "Change a graphic's symbol based on the camera's proximity to it." }
    var snippets: [String] { ["StylePointWithDistanceCompositeSceneSymbolView.swift"] }
    var tags: Set<String> { ["3D", "data", "graphic"] }
    var hasDependencies: Bool { true }

    func makeBody() -> AnyView { .init(StylePointWithDistanceCompositeSceneSymbolView()) }
}

struct StylePointWithPictureMarkerSymbols: Sample {
    var name: String { "Style point with picture marker symbols" }
    var category: String { "Visualization" }
    var description: String { "Use pictures for markers." }
    var snippets: [String] { ["StylePointWithPictureMarkerSymbolsView.swift"] }
    var tags: Set<String> { ["graphics", "marker", "picture", "symbol", "visualization"] }
    
    func makeBody() -> AnyView { .init(StylePointWithPictureMarkerSymbolsView()) }
}

struct StylePointWithSceneSymbol: Sample {
    var name: String { "Style point with scene symbol" }
    var category: String { "Scenes" }
    var description: String { "Show various kinds of 3D symbols in a scene." }
    var snippets: [String] { ["StylePointWithSceneSymbolView.swift"] }
    var tags: Set<String> { ["3D", "cone", "cube", "cylinder", "diamond", "geometry", "graphic", "graphics overlay", "pyramid", "scene", "shape", "sphere", "symbol", "tetrahedron", "tube", "visualization"] }
    
    func makeBody() -> AnyView { .init(StylePointWithSceneSymbolView()) }
}

struct StyleSymbolsFromMobileStyleFile: Sample {
    var name: String { "Style symbols from mobile style file" }
    var category: String { "Visualization" }
    var description: String { "Combine multiple symbols from a mobile style file into a single symbol." }
    var snippets: [String] { ["StyleSymbolsFromMobileStyleFileView.swift", "StyleSymbolsFromMobileStyleFileView.SymbolOptionsListView.swift"] }
    var tags: Set<String> { ["advanced symbology", "mobile style", "multilayer", "stylx"] }
    var hasDependencies: Bool { true }

    func makeBody() -> AnyView { .init(StyleSymbolsFromMobileStyleFileView()) }
}

struct TraceUtilityNetwork: Sample {
    var name: String { "Trace utility network" }
    var category: String { "Utility Networks" }
    var description: String { "Discover connected features in a utility network using connected, subnetwork, upstream, and downstream traces." }
    var snippets: [String] { ["TraceUtilityNetworkView.swift", "TraceUtilityNetworkView.Enums.swift", "TraceUtilityNetworkView.Model.swift", "TraceUtilityNetworkView.Views.swift"] }
    var tags: Set<String> { ["condition barriers", "downstream trace", "network analysis", "subnetwork trace", "toolkit", "trace configuration", "traversability", "upstream trace", "utility network", "validate consistency"] }
    
    func makeBody() -> AnyView { .init(TraceUtilityNetworkView()) }
}

struct ValidateUtilityNetworkTopology: Sample {
    var name: String { "Validate utility network topology" }
    var category: String { "Utility Networks" }
    var description: String { "Demonstrates the workflow of getting the network state and validating the topology of a utility network." }
    var snippets: [String] { ["ValidateUtilityNetworkTopologyView.swift", "ValidateUtilityNetworkTopologyView.Model.swift", "ValidateUtilityNetworkTopologyView.Views.swift"] }
    var tags: Set<String> { ["dirty areas", "edit", "network topology", "online", "state", "trace", "utility network", "validate"] }
    
    func makeBody() -> AnyView { .init(ValidateUtilityNetworkTopologyView()) }
}
